package service

import (
	"fmt"
	"iot_manager_service/app/device/dao"
	"iot_manager_service/app/device/model"
	"iot_manager_service/util"
	"time"
)

var GarbageService = new(garbageService)

type garbageService struct{}

func (s *garbageService) Get(id int) (*dao.Garbage, *util.Errors) {
	// 创建查询实例
	device := &dao.Garbage{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	return device, nil

}

func (s *garbageService) CreateOrUpdate(req dao.Garbage) *util.Errors {
	// 创建查询实例
	device := req
	device.UpdateUser = "TODO" // todo: 使用登录态
	device.UpdateTime = time.Now()
	if req.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = "TODO" // todo: 使用登录态
		if device.IsExistedBySN() {
			fmt.Printf("Create GetDeviceID err \n")
			return util.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
		}
		if err := device.Create(); err != nil {
			fmt.Printf("Create err = %s \n", err.Error())
			return util.FailResponse(err.Error(), nil)
		}
		return util.SuccessResponse(util.Succeeded, nil)
	}
	if device.IsExistedByNameAndCode() {
		return util.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
	}

	if err := device.Update(); err != nil {
		fmt.Printf("Update err = %s \n", err.Error())
		return util.FailResponse(err.Error(), nil)
	}

	//todo operation record
	return util.SuccessResponse(util.Succeeded, nil)
}
func (s *garbageService) List(searchValue string, current, size int) ([]dao.Garbage, *util.Errors) {
	device := dao.Garbage{}
	if searchValue != "" {
		device.DeviceSN = searchValue
		device.DeviceName = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	return devices, nil
}
func (s *garbageService) Remove(id int) *util.Errors {
	// 创建查询实例
	device := &dao.Garbage{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: "TODO", // todo 使用登录态
		UpdateTime: time.Now(),
	}
	//todo
	// service.garbageWayService.CountRelation()

	//todo operation record
	err := device.Delete()
	if err != nil {
		return util.FailResponse(err.Error(), nil)
	}
	return nil
}

func (s *garbageService) GetList() ([]*dao.Garbage, *util.Errors) {
	var devices []*dao.Garbage
	err := util.Redis.Get(getGarbageListRedisKey("000000")).Scan(&devices)
	if err == nil {
		return devices, nil
	}
	device := &dao.Garbage{
		TenantId:  "000000", // todo 使用登录态
		IsDeleted: 0,
	}
	devices, err = device.GetAllDevices()
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	_ = util.Redis.Set(getAlarmListRedisKey("000000"), devices, 0).Err()
	return devices, nil
}

func getGarbageListRedisKey(tenantId string) string {
	return fmt.Sprintf(model.GarbageList, tenantId)
}
